import termcolor
import cv2,mediapipe,time,keyboard,numpy
import pytesseract.pytesseract
from pyzbar.pyzbar import decode as qr_decode_dat
from .parse import Hand,Pose,Face,FaceMesh,ScannedText,qr
import numpy as np,imutils
class Utils:
    def absa(c1:tuple,c2:tuple,t:int):
        """
        Gets two tuples and looks if the (1) values are
        about the same, by subtracting the smaller one from
        the bigger one and looking if the bigger one is bigger
        than t.
        """
        thresh = t
        C1,C2 = c1[1],c2[1]
        if C1 > C2:
            F1 = C1 - C2
            if F1 < thresh:return True
            else:return False
        else:
            F1 = C2 - C1
            if F1 < thresh: return True
            else:return False
class HandDetector:
    def __init__(self,mode=False,maxHands=2,detectionConfidence=0.5,trackConfidence=0.5):
        """
        Sets all the values for mediapipe and the other HandDetector functions.
        """
        self.mode = mode
        self.maxHands = maxHands
        self.detectConf = detectionConfidence
        self.trackConf = trackConfidence
        self.sol = mediapipe.solutions
        self.mpHands = self.sol.hands
        self.hands = self.mpHands.Hands(self.mode,self.maxHands,self.detectConf,self.trackConf)
        self.mpDraw = self.sol.drawing_utils
        self.nt_list = [(0, 0), (0, 0), (0, 0), (0, 0), (0, 0), (0, 0), (0, 0), (0, 0), (0, 0), (0, 0), (0, 0), (0, 0),(0, 0), (0, 0), (0, 0), (0, 0), (0, 0), (0, 0), (0, 0), (0, 0), (0, 0)]
    def get_Hands(self,img):
        """
        Finds the hands img the given img, needs RGB img
        to find the hands, so it first converts them.
        Returns the Hand object with all the landmarks for each hand.
        !!! ONLY INITIALIZE THIS ONCE!!!
        """
        imgRGB = cv2.cvtColor(img,cv2.COLOR_BGR2RGB)
        res = self.hands.process(imgRGB)
        HAND = 0
        if res.multi_hand_landmarks:
            for handLms in res.multi_hand_landmarks:
                HAND = HAND + 1
                List = []
                for id, lm in enumerate(handLms.landmark):
                    h, w, c = img.shape
                    global _COMP_y_cord
                    _COMP_y_cord = lm.z
                    print(_COMP_y_cord)
                    cx, cy = int(lm.x * w), int(lm.y * h)
                    List.append((cx,cy))
                yield Hand(HAND,List,handLms)
        else:
            return Hand(1,self.nt_list,[])
    def draw_hand(self,img,hand:Hand):
        """
        Draws the Landmarks and connections on the image.
        """
        self.mpDraw.draw_landmarks(img,hand.hand_lms ,self.mpHands.HAND_CONNECTIONS)
        return True
class PoseDetector:
    def __init__(self,static_image_mode=False,model_complexity=1,smooth_landmarks=True,min_detection_confidence=0.5,min_tracking_confidence=0.5):
        """
        Sets all the values for mediapipe and the other PoseDetector functions.
        !!! ONLY INITIALIZE THIS ONCE!!!
        """
        self.static_image_mode = static_image_mode
        self.model_complexity = model_complexity
        self.smooth_landmarks = smooth_landmarks
        self.min_detection_conf = min_detection_confidence
        self.min_tracking_conf = min_tracking_confidence
        self.sol = mediapipe.solutions
        self.mpPose = self.sol.pose
        self.pose = self.mpPose.Pose(self.static_image_mode, model_complexity, smooth_landmarks, min_detection_confidence, min_tracking_confidence)
        self.nt_list = [(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),(0,0),]
        self.mpDraw = self.sol.drawing_utils
    def get_Pose(self,img,wd=True):
        """
        Transforms the img to RGB and then builds the Pose object
        based off all the landmarks on the frame.
        Returns the Pose object with the complete list of landmarks.
        """
        imgRGB = cv2.cvtColor(img,cv2.COLOR_BGR2RGB)
        res = self.pose.process(imgRGB).pose_landmarks
        if res:
            List = []
            if wd:
                for id, lm in enumerate(res.landmark):
                    h, w, c = img.shape
                    cx, cy = int(lm.x * w), int(lm.y * h)
                    List.append((cx,cy))
                try:
                    yield Pose(List, res)
                except IndexError:
                    return Pose(self.nt_list, [])
            else:
                yield Pose(self.nt_list, res)
        else:
            return Pose(self.nt_list,[])
    def draw_pose(self,img,pose:Pose):
        """
        Draws the Landmarks and connections on the image.
        """
        self.mpDraw.draw_landmarks(img,pose.results,self.mpPose.POSE_CONNECTIONS)
        return True
class FaceMeshDetector:
    def __init__(self,static_image_mode=False,max_num_faces=1,min_detection_confidence=0.5,min_tracking_confidence=0.5):
        """
        Sets all the values for mediapipe and the other PoseDetector functions.
        !!! ONLY INITIALIZE THIS ONCE!!!
        """
        self.static_image_mode = static_image_mode
        self.max_num_faces = max_num_faces
        self.min_detection_conf = min_detection_confidence
        self.min_tracking_conf = min_tracking_confidence
        self.sol = mediapipe.solutions
        self.mpDraw = self.sol.drawing_utils
        self.mpFaceMesh = self.sol.face_mesh
        self.face_mesh = self.mpFaceMesh.FaceMesh()
        self.drawSpec = self.mpDraw.DrawingSpec(thickness=1, circle_radius=2)
    def get_faces(self,img):
        """
        Transforms the img to RGB and then builds the FaceMesh object
        based off all the landmarks on the frame.
        Returns the FaceMesh object with the complete list of landmarks.
        """
        imgRGB = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        res = self.face_mesh.process(imgRGB)
        if res.multi_face_landmarks:
            for faceLms in res.multi_face_landmarks:
                List = []
                c = -1
                for lm in faceLms.landmark:
                    c = c + 1
                    ih, iw, ic = img.shape
                    cx, cy = int(lm.x * iw), int(lm.y * ih)
                    List.append(((cx,cy),c))
                yield FaceMesh(List,faceLms)
    def draw_mesh(self,img,face_mesh:FaceMesh):
        """
        Draws the Landmarks and connections on the image.
        """
        self.mpDraw.draw_landmarks(img,face_mesh.fLms,self.mpFaceMesh.FACE_CONNECTIONS,self.drawSpec,self.drawSpec)
        return True
class FaceDetection:
    def __init__(self,min_detection_confidence=0.5, model_selection=1):
        self.sol = mediapipe.solutions
        self.mpFaceDetection = self.sol.face_detection
        self.mpDraw = self.sol.drawing_utils
        self.min_detection_conf = min_detection_confidence
        self.model_selection = model_selection
        self.FaceDetection = self.mpFaceDetection.FaceDetection(self.min_detection_conf,self.model_selection)
    def get_faces(self,img):
        imgRGB = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        res = self.FaceDetection.process(imgRGB)
        if res.detections:
            for id, detections in enumerate(res.detections):
                iw, ih, ic = img.shape
                score = int(float(detections.score[0]) * 100)
                ld = detections.location_data
                rbb = ld.relative_bounding_box
                bbox = int(rbb.xmin * iw)+55, int(rbb.ymin * ih)-70, int(rbb.width * iw)+50, int(rbb.height * ih)-20
                List = []
                c = 0
                for kp in ld.relative_keypoints:
                    c = c + 1
                    cx, cy = int(kp.x * iw)+85, int(kp.y * ih)-50
                    cent = (cx, cy)
                    List.append((cent,c))
                yield Face(List,res,score,bbox)
    def draw_face(self,img,face:Face):
        cv2.rectangle(img,face.bbox,color=(0,0,255),thickness=3)
        for lm in face.lms:
            cv2.circle(img,(lm[0],lm[1]),1,(0,0,255),1)
        cv2.putText(img,f'{face.conf_score}%',(face.bbox[0],face.bbox[1]),cv2.FONT_HERSHEY_PLAIN,2,(0,0,255),2)
class Gestures:
    """
    Looks at the bottom of the finger and on the tip, if
    the tip is higher, it returns true. Ont the thumb it looks
    at the bottom and the point above, if both are about the same
    it returns true.

    The locked gestures (ROCK,SCISSOR,PAPER) look at the fingers up and
    returns true if the right fingers are up/down.
    """
    def __init__(self,hand:Hand):
        self.hand = hand
    def INDEX_finger_up(self):
        L1 = self.hand.INDEX
        if L1[0][1] > L1[3][1]:return True
        else:return False
    def THUMB_finger_up(self):
        L1 = self.hand.THUMB
        if L1[0][1] > L1[3][1] and not Utils.absa(L1[0],L1[1],22):
            return True
        else:
            return False
    def PINKY_finger_up(self):
        L1 = self.hand.PINKY
        if L1[0][1] > L1[3][1]:return True
        else:return False
    def RING_finger_up(self):
        L1 = self.hand.RING
        if L1[0][1] > L1[3][1]:return True
        else:return False
    def MIDDLE_finger_up(self):
        L1 = self.hand.MIDDLE
        if L1[0][1] > L1[3][1]:return True
        else:return False
    def paper(self):
        if self.INDEX_finger_up() and self.RING_finger_up() and self.PINKY_finger_up() and self.MIDDLE_finger_up():
            return True
        else:
            return False
    def rock(self):
        if self.INDEX_finger_up() or self.RING_finger_up() or self.PINKY_finger_up() or self.MIDDLE_finger_up():
            return False
        else:
            return True
    def scissor(self):
        if self.INDEX_finger_up() and self.MIDDLE_finger_up() and not self.RING_finger_up() and not self.PINKY_finger_up():
            return True
        else:
            return False
class MixDetection:
    def __init__(self,img):
        self.img = img
    def Detect_License_Placte(self):
        img = cv2.resize(self.img, (600, 400))
        gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
        gray = cv2.bilateralFilter(gray, 13, 15, 15)
        edged = cv2.Canny(gray, 30, 200)
        contours = cv2.findContours(edged.copy(), cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
        contours = imutils.grab_contours(contours)
        contours = sorted(contours, key=cv2.contourArea, reverse=True)[:10]
        screenCnt = None
        for c in contours:
            peri = cv2.arcLength(c, True)
            approx = cv2.approxPolyDP(c, 0.018 * peri, True)
            if len(approx) == 4:
                screenCnt = approx
                break
        if screenCnt is None:
            detected = 0
            return [None,None]
        else:
            detected = 1
        if detected == 1:
            cv2.drawContours(img, [screenCnt], -1, (0, 0, 255), 3)
        try:
            mask = np.zeros(gray.shape, np.uint8)
            new_image = cv2.drawContours(mask, [screenCnt], 0, 255, -1, )
            self.new_image = cv2.bitwise_and(img, img, mask=mask)
            (x, y) = np.where(mask == 255)
            (topx, topy) = (np.min(x), np.min(y))
            (bottomx, bottomy) = (np.max(x), np.max(y))
            Cropped = gray[topx:bottomx + 1, topy:bottomy + 1]
            self.Cropped = cv2.resize(Cropped, (400, 200))
        except:
            return [None,None]
        return [self.Cropped,self.new_image]
class Segmentation:
    def __init__(self,rep_img,rem_perc:float,model_sel=0,body=False):
        """
        rep_img is the replacement for the background.
        rem_perc is the percentage of the background that gets deleted (0.85 is good)
        bod is if the foreground or the background should be replaced
        """
        self.rem = rem_perc
        self.replace_img = rep_img
        self.mode_sel = model_sel
        self.bod = body
        self.model = mediapipe.solutions.selfie_segmentation.SelfieSegmentation(self.mode_sel)
    def segment(self,img):
        """
        The given background Image must be the same size as the normal Image
        """
        imgRGB = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        results = self.model.process(imgRGB) # Create a black-white image
        if self.bod:
            condition = numpy.stack((results.segmentation_mask,) * 3, axis=-1) < self.rem
        else:                                                                               # Apply rem_perc
            condition = numpy.stack((results.segmentation_mask,) * 3, axis=-1) > self.rem
        if isinstance(self.replace_img, tuple):
            _imgBg = numpy.zeros(img.shape, dtype=numpy.uint8)
            _imgBg[:] = self.replace_img                             # Make the background
            imgOut = numpy.where(condition, img, _imgBg)
        else:
            imgOut = numpy.where(condition, img, self.replace_img) # Combine both
        return imgOut
class TextRecognition:
    def __init__(self,img,tesseract_dir:str=None):
        if not tesseract_dir:
            try:
                import tesseract_pack
                self.dat_here = tesseract_pack.data_here
            except ModuleNotFoundError:
                print(termcolor.colored('tesseract-ocr is needed for TextRecognition, install "tesseract-ocr-data" or specify "tesseract_dir"'))
                return
        else:
            self.dat_here = tesseract_dir
        self.def_img = img
        self.IMG = cv2.cvtColor(img,cv2.COLOR_BGR2RGB)
        self.shape = self.IMG.shape
        tmp_hIMG,tmp_wImg,ust = self.shape
        self.hIMG = tmp_hIMG
        self.wIMG =tmp_wImg
        tox = '\\'
        self.install = (f'{self.dat_here.replace(tox,"/")}\\tesseract.exe')
    def get_data(self):
        pytesseract.pytesseract.tesseract_cmd = self.install
        result1 = pytesseract.pytesseract.image_to_boxes(self.IMG)
        result2 = pytesseract.pytesseract.image_to_string(self.IMG)
        obj = ScannedText(result1,result2,self.hIMG,self.wIMG)
        return obj
    def draw_all(self,obj:ScannedText):
        img = self.def_img
        for b in obj.boxes:
            cv2.rectangle(img,(b[0],obj.height-b[1]),(b[2],obj.height-b[3]),(0,0,255),3)
        return img
class qrbar_code:
    def __init__(self,img):
        self.raw_ = qr_decode_dat(img)
        if not self.raw_ == []:
            self.object = qr(img,self.raw_)
        else:
            self.object = None
    def qr_shown(self):
        if self.object == None:
            return False
        else:
            return True
    def get_text(self):
        if self.object == None:return
        obj : qr = self.object
        return obj.decoded_txt
    def get_all_data(self):
        if self.object == None: return
        obj: qr = self.object
        return obj
class Examples:
    def x3_test_all(img):
        report = []
        try:
            q = qrbar_code(img)
            q.get_all_data()
            report.append((True, 'Test succeeded'))
        except Exception as ex:
            print(len(report) + 1)
            report.append(str(ex))
        try:
            q = HandDetector()
            q.get_Hands(img)
            report.append((True, 'Test succeeded'))
        except Exception as ex:
            print(len(report) + 1)
            report.append(str(ex))
        try:
            q = FaceDetection()
            q.get_faces(img)
            report.append((True,'Test succeeded'))
        except Exception as ex:
            print(len(report) + 1)
            report.append((False,str(ex)))
        try:
            q = PoseDetector(True)
            q.get_Pose(img)
            report.append((True,'Test succeeded'))
        except Exception as ex:
            print(len(report) + 1)
            report.append((False,str(ex)))
        try:
            q = FaceMeshDetector()
            q.get_faces(img)
            report.append((True,'Test succeeded'))
        except Exception as ex:
            print(len(report) + 1)
            report.append((False,str(ex)))
        try:
            q = Segmentation(img,0.9)
            q.segment(img)
            report.append((True,'Test succeeded'))
        except Exception as ex:
            print(len(report) + 1)
            report.append((False,str(ex)))
        return report

    def SPS(input=1,quit_key='esc'):
        """
        Simple scissor paper rock game. Input is for 'cv2.VideoCapture(input)'
        """
        global a_time,erg
        erg = 0
        class hand_init:
            def zei(hand: Hand):
                g1, g2, g3 = Gestures.paper(hand), Gestures.rock(hand), Gestures.scissor(hand)
                if g1:
                    cv2.putText(img, f'PAPER', (10, 50), cv2.FONT_HERSHEY_PLAIN, 1, (255, 255, 255), 1)
                    return 1
                elif g2:
                    cv2.putText(img, f'ROCK', (10, 50), cv2.FONT_HERSHEY_PLAIN, 1, (255, 255, 255), 1)
                    return 2
                elif g3:
                    cv2.putText(img, f'SCISSOR', (10, 50), cv2.FONT_HERSHEY_PLAIN, 1, (255, 255, 255), 1)
                    return 3
                else:return 4
            def pst(a1: int, a2: int):
                global erg
                if a1 == 4 or a2 == 4:
                    erg = 3
                    return
                if a1 == 1:
                    if a2 == 1:
                        erg = 4
                        return
                    elif a2 == 2:
                        erg = 1
                        return
                    elif a2 == 3:
                        erg = 2
                        return
                elif a1 == 2:
                    if a2 == 2:
                        erg = 4
                        return
                    elif a2 == 3:
                        erg = 1
                        return
                    elif a2 == 1:
                        erg = 2
                        return
                elif a1 == 3:
                    if a2 == 1:
                        erg = 1
                        return
                    elif a2 == 2:
                        erg = 2
                        return
                    elif a2 == 3:
                        erg = 4
                        return
        vid = cv2.VideoCapture(input)
        detector = HandDetector(detectionConfidence=0.75, trackConfidence=0.75)
        a_time = time.time() + 5
        while True:
            _, img = vid.read()
            hands = detector.get_Hands(img)
            c_time = time.time()
            for hand in hands:
                if not erg == 0:
                    if erg == 1:cv2.putText(img, 'PLAYER1 WON SHOW ONLY INDEX FINGER TO RESTART', (20, 50),cv2.FONT_HERSHEY_PLAIN, 2, (255, 255, 255), 2)
                    elif erg == 2:cv2.putText(img, 'PLAYER2 WON SHOW ONLY INDEX FINGER TO RESTART', (20, 50),cv2.FONT_HERSHEY_PLAIN, 2, (255, 255, 255), 2)
                    elif erg == 3:cv2.putText(img, 'WRONG GESTURE SHOW ONLY INDEX FINGER TO RESTART', (20, 50),cv2.FONT_HERSHEY_PLAIN, 2, (255, 255, 255), 2)
                    elif erg == 4:cv2.putText(img, 'NOT SURE SHOW ONLY INDEX FINGER TO RESTART', (20, 50), cv2.FONT_HERSHEY_PLAIN,2, (255, 255, 255), 2)
                    if Gestures.INDEX_finger_up(hand) and not Gestures.RING_finger_up(hand) and not Gestures.PINKY_finger_up(hand) and not Gestures.MIDDLE_finger_up(hand) and not Gestures.THUMB_finger_up(hand):
                        cv2.putText(img, 'RESTARTING', (50, 50), cv2.FONT_HERSHEY_PLAIN, 2, (255, 255, 255), 2)
                        a_time = time.time() + 5
                        erg = 0
                        continue
                elif Utils.absa((0, c_time), (0, a_time), 0.1):
                    if hand.hand_n == 1:
                        global a1
                        a1 = hand_init.zei(hand)
                    elif hand.hand_n == 2:a2 = hand_init.zei(hand)
                elif erg == 0:cv2.putText(img, f'STARTING IN : {str(a_time - c_time)}', (20, 50), cv2.FONT_HERSHEY_PLAIN, 2,(255, 255, 255), 2)
                cv2.putText(img, str(hand.hand_n), hand.WRIST, cv2.FONT_HERSHEY_PLAIN, 3, (255, 255, 255), 2)
            if Utils.absa((0, c_time), (0, a_time), 0.1):
                try:hand_init.pst(a1, a2)
                except NameError:erg = 0
            cv2.imshow("Image", img)
            if keyboard.is_pressed(quit_key):return
            cv2.waitKey(1)
    def face_detection(input=1,quit_key='esc'):
        cap = cv2.VideoCapture(input)
        pTime = time.time()
        detector = FaceDetection()
        while True:
            _, img = cap.read()
            faces = detector.get_faces(img)
            for face in faces:
                for lm in face.lms:
                    cv2.putText(img,f'{str(face.ext_lms[1][1])}',lm,cv2.FONT_HERSHEY_PLAIN,2,(0,0,255),2)
                detector.draw_face(img, face)
            cTime = time.time()
            fps = 1 / (cTime - pTime)
            cv2.putText(img, f'FPS : {str(int(fps))}', (10, 50), cv2.FONT_HERSHEY_PLAIN, 2, (255, 0, 0), 2)
            pTime = cTime
            cv2.imshow("Image", img)
            if keyboard.is_pressed(quit_key):
                return
            cv2.waitKey(1)
    def bg_removal(rep_img:str,rep_perc:float,input=1,quit_key='esc'):
        cap = cv2.VideoCapture(input)
        rep_img_ = cv2.imread(rep_img)
        seg = Segmentation(rep_img_,rep_perc)
        while True:
            _, img = cap.read()
            ref_img = seg.segment(img)
            cv2.imshow("Image", ref_img)
            if keyboard.is_pressed(quit_key):
                return
            cv2.waitKey(1)


import pygame
from pydub import AudioSegment
from .errors import *

class Player:
    def __init__(self):
        """
        Uses pygame to mainly play music.
        All functionalities should be self explaining.
        """
        pygame.init()
        pygame.mixer.init()
        self._music = pygame.mixer.music
        self._loaded_file = None
    def play(self,file:str,*,auto_format:bool=True,loops:int=0,start:float=0.0,fade:int=0):
        self.loadIn(file,auto_format)
        self._music.play(loops,start,fade)
        self.volume(0.5)
    def rewind(self):
        self._music.rewind()
    def stop(self):
        self._music.stop()
    def loadIn(self,file:str,auto_format):
        self._loaded_file = file
        if auto_format:
            nfn, t = (str(self._loaded_file).split('.', self._loaded_file.count('.')))
            f = Format(self._loaded_file)
            f.export(nfn + ".mp3")
            self._loaded_file = nfn + ".mp3"
        try:
            self._music.load(self._loaded_file)
        except pygame.error:
            raise FormatError("The Format of the File must be 'mp3'. Use 'auto_format' to automatically make it an 'mp3' file.")
    def volume(self,vol:float):
        self._music.set_volume(vol)
    def get_pos(self):
        return int(self._music.get_pos())
    def pause(self):
        self._music.pause()
    def fade(self,fade:int):
        self._music.fadeout(fade)
    def queueUp(self,file):
        self._music.queue(file)
    def loadOut(self):
        self._music.unload()
    def get_volume(self):
        return self._music.get_volume()
    def busy(self):
        return self._music.get_busy()
    def set_volume(self,volume:int):
        """
        :param volume: Milliseconds
        """
        self._music.set_volume(volume)
    def set_pos(self,pos:float):
        self._music.set_pos(int(str(pos)))
    def resume(self):
        self._music.unpause()
    def _collect(self):
        collection = {
            'volume' : self.get_volume(),
            'file' : self._loaded_file,
            'position' : self.get_pos()
        }
        return collection
    def __str__(self):
        return str(self._collect())
    def __len__(self):
        return int(self.get_pos())

class Format:
    def __init__(self,file:str,*,format:str=None):
        """
        Mainly formats audio, using AudioSegment from pydub.
        :param file:
        This is were the Audio is taken from
        :param format:
        The format that the file should be in, leave blank
        if the requested format is the same as the existing
        format.
        """
        self._file = file
        self._audio = AudioSegment.from_file(file, format=format)
    def export(self,filename:str,*,format:str="mp3"):
        self._audio.export(filename,format)
    def reverse(self):
        self._audio.reverse()
    def empty(self):
        self._audio.empty()
    def load_file(self,file:str,*,format:str=None):
        self._file = file
        self._audio = AudioSegment.from_file(file, format=format)
    def __str__(self):
        return str(self._file)
    def __len__(self):
        return len(self.__str__())


import multiprocessing,os,mediapipe
import httpx as x

__ext_DCF__ = False
class using:
    def __init__(self):
        pass
    def create_process(self,target,args):
        p = multiprocessing.Process(target=target,args=args)
        return p
    def start_process(self,Process):
        global _mp_tkt_process
        Process.start()
        _mp_tkt_process=Process
    def pull_values(self,V:str):
        t,hex=V.split('{"value":"',1)
        hex,t=hex.split('","clean":"',1)
        t,rgb=V.split('"value":"rgb',1)
        rgb,t=rgb.split('"},"hsl":{"',1)
        t,hsl=V.split('"value":"hsl',1)
        hsl,t=hsl.split('"},"hsv":{"fraction"',1)
        t,hsv=V.split(',"value":"hsv',1)
        hsv,t=hsv.split('","h":',1)
        return (hex,rgb,hsl,hsv,V)
    def del_files(self,dat: str,log:bool):
        for I in os.listdir(dat):
            E = (dat + "/" + I)
            if log == True:print(f'Deleting : {E}')
            if os.path.isfile(E):
                os.remove(E)
            elif os.path.isdir(E):
                self.del_files(E,log)

    def del_folders(self,dat: str,log:bool):
        for I in os.listdir(dat):
            E = (dat + "/" + I)
            if log == True:print(f'Deleting : {E}')
            if os.path.isdir(E):
                try:
                    os.rmdir(E)
                except OSError:
                    self.del_folders(E,log)
                    os.rmdir(E)
        os.rmdir(dat)
class Q:
    build_main = 'https://www.thecolorapi.com/'
    build_hex = 'id?hex='
    build_rgb = 'id?rgb=rgb'
    urlSafeChars=".","\\","(",")","{","[","|","]","}","^",">","<"
class ScannedText:
    def __init__(self,res1,res2,img_dat1,img_dat2):
        self.ext_text = res2
        self.raw_boxes = res1
        self.height = img_dat2
        self.width = img_dat1
        I = []
        for b in self.raw_boxes.splitlines():
            b = b.split(' ')
            i = int(b[1]),int(b[2]),int(b[3]),int(b[4]),str(b[0])
            I.append(i)
        self.boxes = I
class Hand:
    def __init__(self,n_hand:int,lm:list,handLms):
        self.hand_n             = n_hand
        self.sol                = mediapipe.solutions
        self.lm_list            = lm
        self.hand_lms           = handLms
        self.mpHands            = self.sol.hands
        self.WRIST              = self.lm_list[0]
        self.THUMB_CMC          = self.lm_list[1]
        self.THUMB_MCP          = self.lm_list[2]
        self.THUMB_IP           = self.lm_list[3]
        self.THUMB_TIP          = self.lm_list[4]
        self.INDEX_FINGER_MCP   = self.lm_list[5]
        self.INDEX_FINGER_PIP   = self.lm_list[6]
        self.INDEX_FINGER_DIP   = self.lm_list[7]
        self.INDEX_FINGER_TIP   = self.lm_list[8]
        self.MIDDLE_FINGER_MCP  = self.lm_list[9]
        self.MIDDLE_FINGER_PIP  = self.lm_list[10]
        self.MIDDLE_FINGER_DIP  = self.lm_list[11]
        self.MIDDLE_FINGER_TIP  = self.lm_list[12]
        self.RING_FINGER_MCP    = self.lm_list[13]
        self.RING_FINGER_PIP    = self.lm_list[14]
        self.RING_FINGER_DIP    = self.lm_list[15]
        self.RING_FINGER_TIP    = self.lm_list[16]
        self.PINKY_MCP          = self.lm_list[17]
        self.PINKY_PIP          = self.lm_list[18]
        self.PINKY_DIP          = self.lm_list[19]
        self.PINKY_TIP          = self.lm_list[20]
        self.THUMB              = [self.THUMB_CMC,self.THUMB_MCP,self.THUMB_IP,self.THUMB_TIP]
        self.INDEX              = [self.INDEX_FINGER_MCP,self.INDEX_FINGER_PIP,self.INDEX_FINGER_DIP,self.INDEX_FINGER_TIP]
        self.MIDDLE             = [self.MIDDLE_FINGER_MCP,self.MIDDLE_FINGER_PIP,self.MIDDLE_FINGER_DIP,self.MIDDLE_FINGER_TIP]
        self.RING               = [self.RING_FINGER_MCP,self.RING_FINGER_PIP,self.RING_FINGER_DIP,self.RING_FINGER_TIP]
        self.PINKY              = [self.PINKY_MCP,self.PINKY_PIP,self.PINKY_DIP,self.PINKY_TIP]
class Pose:
    def __init__(self,lm_list:list,res):
        self.lm_list          = lm_list
        self.results          = res
        self.nose             = lm_list[0]
        self.left_eye_inner   = lm_list[1]
        self.left_eye         = lm_list[2]
        self.left_eye_outer   = lm_list[3]
        self.right_eye_inner  = lm_list[4]
        self.right_eye        = lm_list[5]
        self.right_eye_outer  = lm_list[6]
        self.left_ear         = lm_list[7]
        self.right_ear        = lm_list[8]
        self.mouth_left       = lm_list[9]
        self.mouth_right      = lm_list[10]
        self.left_shoulder    = lm_list[11]
        self.right_shoulder   = lm_list[12]
        self.left_elbow       = lm_list[13]
        self.right_elbow      = lm_list[14]
        self.left_wrist       = lm_list[15]
        self.right_wrist      = lm_list[16]
        self.left_pinky       = lm_list[17]
        self.right_pinky      = lm_list[18]
        self.left_index       = lm_list[19]
        self.right_index      = lm_list[20]
        self.left_thumb       = lm_list[21]
        self.right_thumb      = lm_list[22]
        self.left_hip         = lm_list[23]
        self.right_hip        = lm_list[24]
        self.left_knee        = lm_list[25]
        self.right_knee       = lm_list[26]
        self.left_ankle       = lm_list[27]
        self.right_ankle      = lm_list[28]
        self.left_heel        = lm_list[29]
        self.right_heel       = lm_list[30]
        self.left_foot_index  = lm_list[31]
        self.right_foot_index = lm_list[32]
class FaceMesh:
    def __init__(self,lm_list:list,faceLms):
        # There is no landmark index on mediapipe.dev
        List = []
        for l in lm_list:
            List.append((l[0]))
        self.lms = List
        self.lms_ext = lm_list
        self.fLms = faceLms
class Face:
    def __init__(self,lm_list:list,res,score:int,bbox):
        # There is no landmark index on mediapipe.dev
        List = []
        for l in lm_list:
            List.append((l[0]))
        self.lms = List
        self.results = res
        self.conf_score = score
        self.bbox = bbox
        self.ext_lms = lm_list
class qr:
    def __init__(self,img,decoded):
        self.img = img
        self.raw_decoded = decoded[0]
        self.decoded_txt = self.raw_decoded.data.decode('utf-8')
        self.qrtype = self.raw_decoded.type
        self.r_rectangle = self.raw_decoded.rect
        self.polygon = self.raw_decoded.polygon
        self.rectangle = self.raw_decoded.rect.left,self.raw_decoded.rect.top,self.raw_decoded.rect.width,self.raw_decoded.rect.height
using = using()


__title__ = 'tkt-Toolkit'
__author__ = 'Suprime'
__license__ = 'MIT'
__copyright__ = 'Copyright 2020-2021 Suprime'
__version__ = '1.9'

import time,socket,json,os,piexif.helper
from . import SpeechRecongition,translator, parse,tracking,audio
from .errors import *
from win32com.client import Dispatch

here = os.path.dirname(os.path.abspath(__file__))

Tracking = tracking
Parse = parse
AudioFormatting = audio.Format
AudioPlayer = audio.Player

class exlist:
    def __init__(self,list:list):
        """
        Some extra life improvements for list type objects.
        :param list:
        The list that is used for all operations
        """
        self._list = list
    def in_list(self,find:str,ignore_lower_and_upper:bool=False):
        c = -1
        for item in self._list:
            c += 1
            if item == find:
                return (True,c)
            if ignore_lower_and_upper:
                if item.lower() == find.lower():
                    return (True,c)
        return (False)
    def del_item(self,pos:int):
        new = []
        c = -1
        for item in self._list:
            c = c + 1
            if c == pos:
                pass
            else:
                new.append(item)
        return new
    def add_item(self,pos:int,entry:str):
        new = []
        c = -1
        for item in self._list:
            c = c + 1
            new.append(item)
            if c == pos:
                new.append(entry)
        return new
    def tuple_array(self,array2:list):
        c1 = 0
        c2 = 0
        array1 = self._list
        array3 = []
        for item1 in array1:
            c1 = c1 + 1
            for item2 in array2:
                c2 = c2 + 1
                tupled = (item1,item2)
                array3.append(tupled)
        return array3
    def __len__(self):
        return len(self._list)
    def __str__(self):
        return str(self._list)

class SpeechRecognition:
    """
    Requires Pyaudio that you have to install
    using 'pipwin install pyaudio'.
    Records or reads audio and transforms it to text.
    Can also list Microphones.
    """
    def __init__(self,show_msg:bool=True):
        self.show_msg = show_msg
    def from_Microphone(self):
        return SpeechRecongition.recon.recon_from_mic(self.show_msg)
    def from_Wav_File(self,wav_file: str, show_msg: bool = True):
        return SpeechRecongition.recon.recon_from_file(wav_file, show_msg)
    def list_Microphones(self):
        for EE in SpeechRecongition.recon.get_mics(self.show_msg): yield EE

class Toolkit:
    def __init__(self):
        """
        The main function with all sorts of things.
        """
        self.__version__ = __version__
    def __str__(self):
        return str("Version : " + self.__version__)
    def del_all(self,dir : str):
        """
        Clears out all of the files in the folder and
        then delets them.
        :param dir:
        The target folder.
        :return:
        None
        """
        parse.using.del_files(dir,False)
        parse.using.del_folders(dir,False)
        return None
    def getip(self):
        """
        Gets the users IP using the python
        built-in socket package.
        """
        return socket.gethostbyname(socket.gethostname())
    def translate(self,Text:str,Dest,From='auto'):
        """
        Translates text into another language using
        the googletrans package.
        :param Text:
        The Text that is going to be translated.
        :param Dest:
        The destination language for the text.
        :param From:
        The language of the given text.
        Use 'auto' here to automatically recognize
        the language.
        :return:
        Returns 2 values:
        1 : Translated Text
        2 : Tuple:
            1 : Text Language
            2 : Translated Language
            3 : Text
        """
        return translator.Translate(Text,Dest,From)
    def weather(self,city:str, key:str):
        """
        Gets the weather from 'openweathermap.org',
        uses the standard free to use api.
        :param city:
        The city that the weather is going
        to be from.
        :param key:
        The key for 'openweathermap.org'.
        :return:
        """
        api_key = key
        uul = ("https://api.openweathermap.org/data/2.5/weather?q=" + city + "&appid=" + api_key)
        url = uul
        response = parse.x.get(url).text
        data = json.loads(response)
        return data
    def get_color(self,rgb : tuple = None, hex : int = None):
        """
        Gets the color-data of one 'hex'/'rgb' color.
        At least one of the two parameters has to be entered.
        :param rgb:
        The rgb color code tuple.
        :param hex:
        The hex color value. (int)
        :return:
        Returns 5 values:
        1 : The hex value.
        2 : The rgb tuple.
        3 : The hsl code.
        4 : The vsv code.
        5 : All of them.
        """
        if rgb == None and hex == None:
            raise ValueError('Not enough values.')
        elif not rgb == None and not hex == None:
            raise ValueError('Too many values.')
        elif not rgb == None and hex == None:
            R = 1
        elif rgb == None and not hex == None:
            R = 2
        else:
            raise InternalError
        if R == 1:
            req = parse.Q.build_main + parse.Q.build_rgb + str(rgb)
        elif R == 2:
            req = parse.Q.build_main + parse.Q.build_hex + str(hex.replace('#', '', 1))
        else:
            raise InternalError
        resp = (parse.x.get(req).text)
        return parse.using.pull_values(resp)
    def calc(self,calc):
        exec(f'global __COMP_CALC_END__\n__COMP_CALC_END__=({calc})')
        return __COMP_CALC_END__
    def do_for(self,n_seconds: int, func):
        """
        Runs the given function for the given amount of seconds.
        :param n_seconds:
        The seconds the function is run for.
        :param func:
        The function that is ran.
        :return:
        None
        """
        t_end = time.time() + n_seconds
        while time.time() < t_end: func()
        return None
    def read_metadata(self,filename:str):
        """
        Reads the metadata from an image file.
        :param filename:
        The file to be read.
        :return:
        The metadata in json format.
        """
        exif_dict = piexif.load(filename)
        user_comment = piexif.helper.UserComment.load(exif_dict["Exif"][piexif.ExifIFD.UserComment])
        d = json.loads(user_comment)
        return d
    def write_metadata(self,filename:str,json_data:dict):
        """
        Writes metadata into an image file.
        :param filename:
        The image to be used.
        :param json_data:
        The data to be written.
        :return:
        None
        """
        exif_dict = piexif.load(filename)
        exif_dict["Exif"][piexif.ExifIFD.UserComment] = piexif.helper.UserComment.dump(
            json.dumps(json_data),
            encoding="unicode"
        )
        piexif.insert(
            piexif.dump(exif_dict),
            filename
        )
        return None
    def create_shortcut(self,target:str,output:str):
        """
        Creates a shortcut on windows.
        :param target:
        The file that the shortcut accesses.
        :param output:
        The output file.
        :return:
        None
        """
        path = output
        shell = Dispatch('WScript.Shell')
        shortcut = shell.CreateShortCut(path)
        shortcut.Targetpath = target
        shortcut.WorkingDirectory = target
        shortcut.IconLocation = target
        shortcut.save()
        return None


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="tkt-Toolkit",
    version="1.9",
    author="Suprime",
    license = 'MIT',
    author_email="suprime.sendings@gmail.com",
    description="Useful collection of python packages and scripts.",
    long_description= long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/MOBSkuchen/tkt-Toolkit",
    packages=setuptools.find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=['httpx','googletrans==3.1.0a0','SpeechRecognition','termcolor','mediapipe','opencv-python','keyboard','pytesseract','pyzbar','piexif','pydub','pygame'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)


class OutOfService(Exception):
    def __init__(self,Note:str=""):
        pass
class InternalError(Exception):
    def __init__(self,Note:str=""):
        pass
class NothingFoundError(Exception):
    def __init__(self,Note:str=""):
        pass
class DummyError(Exception):
    def __init__(self,Note:str=""):
        pass
class NotImplemented(Exception):
    def __init__(self,Note:str=""):
        pass
class NotActivated(Exception):
    def __init__(self,Note:str=""):
        pass
class InvalidAuth(Exception):
    def __init__(self,Note:str=""):
        pass
class NonExsistant(Exception):
    def __init__(self,Note:str=""):
        pass
class IndexOutOfItems(Exception):
    def __init__(self,Note:str=""):
        pass
class ItemNotFound(Exception):
    def __init__(self,Note:str=""):
        pass
class MissingArgument(Exception):
    def __init__(self,Note:str=""):
        pass
class FormatError(Exception):
    def __init__(self,Note:str=""):
        pass


import termcolor
import speech_recognition as sr
r = sr.Recognizer()
class using:
    def get_mics(comp:bool):
        for index, name in enumerate(sr.Microphone.list_microphone_names()):
            if comp ==True:FF = (f'Found microphone: ({name})[{index}]')
            else:FF = (index,name)
            yield FF
    def get_from_file(wav_file:str,msg_on:bool=True):
        if msg_on:print(termcolor.colored(f'Converting {wav_file} in to text.', 'yellow'))
        with sr.AudioFile(wav_file) as source:
            audio_text = r.listen(source)
            try:
                text = r.recognize_google(audio_text)
                return text
            except Exception as ex:
                b = ''
                if msg_on:b = f'Got an error ({ex}), please try again'
                return b
    def get_from_mic(msg_on:bool=True):
        global _end_spr_tkt_END
        try:
            if msg_on: print(termcolor.colored(f'Converting MIC audio to text.', 'yellow'))
            with sr.Microphone() as source:
                aud_text = r.listen(source)
                final_text = r.recognize_google(aud_text)
                b = final_text
        except Exception as ex:
            b = ''
            if msg_on: b = f'Got an error ({ex}), please try again'
        return b
class recon:
    def recon_from_mic(msg:bool=True):
        return using.get_from_mic(msg)
    def recon_from_file(file:str,msg:bool=True):
        return using.get_from_file(msg)
    def get_mics(detailed:bool=False):
        for E in using.get_mics(detailed):yield E


from googletrans import Translator

translator = Translator()
def Translate(text:str,dest,src:str='auto'):
    obj = translator.translate(text,dest,src)
    src_lng = obj.src
    dest_lng = obj.dest
    dest = obj.text
    src = text
    ext = (src_lng,dest_lng,src)
    return (dest,ext)